import pytest
from unittest.mock import patch
from cogniceptshell.configuration import Configuration

@pytest.fixture
def mock_requests_get():
    with patch('cogniceptshell.configuration.requests.get') as mock_get:
        yield mock_get

api_uri = 'https://dev.cognicept.systems/api/web/v2/'

def test_get_robot_id_success(mock_requests_get, capsys):
    mock_response = {
        'robot_id': '12345'
    }
    mock_requests_get.return_value.json.return_value = mock_response

    auth_key = 'test_auth_key'
    robot_code = 'r2'
    org_id = 'cognicept_systems_2'
    
    object = Configuration()
    robot_id = object.get_robot_id(api_uri, auth_key, robot_code, org_id)
    assert robot_id == '12345'

    captured = capsys.readouterr()
    assert "The robot_code input does not match a robot" not in captured.out

def test_get_robot_id_failure(mock_requests_get, capsys):
    mock_response = {
        'message': 'test_message'
    }
    mock_requests_get.return_value.json.return_value = mock_response

    auth_key = 'test_auth_key'
    robot_code = 'r2'
    org_id = 'cognicept_systems_2'
    
    object = Configuration()
    robot_id = object.get_robot_id(api_uri, auth_key, robot_code, org_id)
    assert robot_id is None

    captured = capsys.readouterr()
    assert "The robot_code input does not match a robot" in captured.out

def test_get_robot_id_exception(mock_requests_get, capsys):
    mock_requests_get.side_effect = Exception('API Error')

    auth_key = 'my_auth_key'
    robot_code = 'my_robot_code'
    org_id = 'my_org_id'

    object = Configuration()
    robot_id = object.get_robot_id(api_uri, auth_key, robot_code, org_id)
    assert robot_id is None

    captured = capsys.readouterr()
    assert "Cognicept API Error: API Error" in captured.out
    
def test_get_organization_id_success(mock_requests_get, capsys):
    mock_response = {
        'data': [
            {
                'organization_id': 'org123'
            }
        ]
    }
    mock_requests_get.return_value.json.return_value = mock_response

    auth_key = 'my_auth_key'
    org_code = 'my_org_code'
    
    object = Configuration()
    org_id = object.get_organization_id(api_uri, auth_key, org_code)
    assert org_id == 'org123'

    captured = capsys.readouterr()
    assert "The org_code input does not match any organization" not in captured.out

def test_get_organization_id_failure(mock_requests_get, capsys):
    mock_requests_get.return_value.json.return_value = {'data': []}

    auth_key = 'my_auth_key'
    org_code = 'my_org_code'
    
    object = Configuration()
    org_id = object.get_organization_id(api_uri, auth_key, org_code)
    assert org_id is None

    captured = capsys.readouterr()
    assert "The org_code input does not match any organization" in captured.out

def test_get_organization_id_exception(mock_requests_get, capsys):
    mock_requests_get.side_effect = Exception('API Error')

    auth_key = 'my_auth_key'
    org_code = 'my_org_code'

    object = Configuration()
    org_id = object.get_organization_id(api_uri, auth_key, org_code)
    assert org_id is None

    captured = capsys.readouterr()
    assert "Cognicept API Error: API Error" in captured.out



# coding=utf8
# Copyright 2023 Cognicept Systems
# Author: Kenneth Chow (kenneth.chow@kabam.ai)
# --> RobotActions class handles on demand custom robot actions on cognicept-shell

import getpass
import json
import requests
import jwt
import io 
import base64
from cogniceptshell.common import bcolors
from cogniceptshell.configuration import Configuration
from cogniceptshell.agent_life_cycle import AgentLifeCycle

class RobotActions:
    """
    A class to manage robot
    ...

    Parameters
    ----------
    None

    Methods
    -------
    login(username, password):
        API call to login the user to smart+

    get_organization(org_id, headers):
        API call to retrieve organization details

    get_robot(robot_id, org_id, headers):
        API call to retrieve robot details
    
    create_robot(robot_details, headers):
        API call to create robot

    get_robot_missions(robot_id, headers):
        API call to retrieve missions associated with robot_id
    
    get_mission_instances(mission_id, headers):
        API call to retrieve instances associated with mission_id
    
    create_mission(mission_details, headers):
        API call to create mission
    
    create_waypoint(waypoint_details, headers):
        API call to create waypoint

    get_map(map_id, headers):
        API call to retrieve map 

    create_map(map_details, headers):
        API call to create map

    get_schedules(schedule_id, headers):
        API call to retrieve schedule

    create_schedule(schedule_details, headers):
        API call to create schedule

    get_property_by_id(property_id, headers):
        API call to retrieve property by property_id

    get_property_by_filter(filters, headers):
        API call to retrieve property by filters

    create_property(property_details, headers):
        API call to create property

    switch_org(org_id, headers):
        API call to switch organizations

    rollback(created_data, headers):
        Deletes any created data if an error occurs 
    
    move(args):
        Main entry point for moving robots from one org to another

    """

    def login(self, username, password):
        login_base_uri = self.api_uri + "user/login"
        login_resp = requests.post(login_base_uri, json={"username": str(username), "password": str(password)}, timeout=5)
        if json.loads(login_resp.content.decode()).get('message', '') == "Username and password combination not valid":
            print(bcolors.FAIL + "Error in logging in: Wrong credentials" + bcolors.ENDC)
            raise SystemExit(1)
        return json.loads(login_resp.content.decode())['access_token']

    def mfa_verfication(self, auth_key, headers):
        mfa_verification_base_uri = self.api_uri + "user/mfa/verify"
        otp_trial = 3
        loop = True
        while loop:
            if otp_trial == 0:
                key = auth_key
                loop = False
            else:
                otp = getpass.getpass('OTP from Authenticator: ')
                x = requests.post(mfa_verification_base_uri, headers=headers, json={'otp': otp})
                if 'access_token' not in x.json():
                    print('Invalid OTP! Please try again...')
                else:
                    key = x.json()["access_token"]
                    loop = False
            otp_trial -= 1
        return key
    
    def get_organization_by_id(self, org_id, headers):
        org_base_uri = self.api_uri + "organization/"
        get_org_uri = org_base_uri + '?filter={{"organization_id":"{org_id}"}}'.format(org_id=org_id)
        get_org_resp = requests.get(get_org_uri, headers=headers, timeout=5)
        json_resp = json.loads(get_org_resp.content.decode())
        if get_org_resp.status_code != 200 or (isinstance(json_resp.get('data', ''), dict) and 'psycopg2.errors' in json_resp['data'].get('Message', '')):
             print(bcolors.FAIL + "Error in getting organization {org_id}".format(org_id=org_id) + bcolors.ENDC)
             raise SystemExit(1)
        elif len(json.loads(get_org_resp.content.decode())['data']) == 0:
             print(bcolors.FAIL + "No such organization {org_id}".format(org_id=org_id) + bcolors.ENDC)
             raise SystemExit(1)
        return json.loads(get_org_resp.content.decode())['data'][0]
    
    def get_organization_by_org_code(self, org_code, headers):
        org_base_uri = self.api_uri + "organization/"
        get_org_uri = org_base_uri + '?filter={{"organization_code":"{org_code}"}}'.format(org_code=org_code)
        get_org_resp = requests.get(get_org_uri, headers=headers, timeout=5)
        if get_org_resp.status_code != 200:
             print(bcolors.FAIL + "Error in getting organization" + bcolors.ENDC)
             raise SystemExit(1)
        elif len(json.loads(get_org_resp.content.decode())['data']) == 0:
             print(bcolors.FAIL + "No such organization {org_code}".format(org_code=org_code) + bcolors.ENDC)
             raise SystemExit(1)           
        return json.loads(get_org_resp.content.decode())['data'][0]

    def get_robot_by_id(self, robot_id, org_id, headers):
        get_robot_uri = self.api_uri + "robot/organization?robot_id={robot_id}&organization_id={org_id}".format(robot_id=robot_id, org_id=org_id)
        get_robot_resp = requests.get(get_robot_uri, headers=headers, timeout=5)
        if get_robot_resp.status_code != 200:
            print(bcolors.FAIL + "Error in retrieving robot" + bcolors.ENDC)
            raise SystemExit(1)
        elif len(get_robot_resp.content.decode()) == 0 or json.loads(get_robot_resp.content.decode()).get('message', '') == 'Bad request. Invalid input':
            print(bcolors.FAIL + "No such robot {robot_id} in organization {org_id}".format(robot_id=robot_id, org_id=org_id) + bcolors.ENDC)
            raise SystemExit(1)
        return json.loads(get_robot_resp.content.decode())

    def get_robot_by_code(self, robot_code, org_code, headers):
        get_robot_uri = self.api_uri + "robot/organization_code?robot_code={robot_code}&organization_code={org_code}".format(robot_code=robot_code, org_code=org_code)
        get_robot_resp = requests.get(get_robot_uri, headers=headers, timeout=5)
        if get_robot_resp.status_code != 200:
             print(bcolors.FAIL + "Error in retrieving robot. Message: {err}".format(err=json.loads(get_robot_resp.content.decode()).get('message', '')) + bcolors.ENDC)
             raise SystemExit(1)
        elif len(json.loads(get_robot_resp.content.decode())) == 0 or json.loads(get_robot_resp.content.decode()).get('message', '') == 'Bad request. Invalid input':
            print(bcolors.FAIL + "No such robot {robot_code} in organization {org_code}".format(robot_code=robot_code, org_code=org_code) + bcolors.ENDC)
            raise SystemExit(1)
        return json.loads(get_robot_resp.content.decode())

    def create_robot(self, robot_details, headers):
        create_robot_uri = self.api_uri + "robot/"
        create_robot_resp = requests.post(create_robot_uri, json=robot_details, headers=headers, timeout=5)
        if create_robot_resp.status_code != 200:
             raise Exception(bcolors.FAIL + "Error in creating robot" + bcolors.ENDC)
        return json.loads(create_robot_resp.content.decode())['robot_id']

    def delete_robot(self, robot_id, organization_id, headers):
        delete_robot_uri = self.api_uri + "robot/organization"
        delete_robot_resp = requests.delete(delete_robot_uri, params={"robot_id": "{robot_id}".format(robot_id=robot_id), "organization_id": "{organization_id}".format(organization_id=organization_id)}, headers=headers, timeout=5)
        if delete_robot_resp.status_code != 200:
             print(bcolors.FAIL + "Error in deleting robot {robot_id}. Please manually delete it".format(robot_id=robot_id) + bcolors.ENDC)
             return None
        return json.loads(delete_robot_resp.content.decode()).get('message', None)

    def get_robot_missions(self, robot_id, headers):
        mission_base_uri = self.api_uri + "mission/"
        get_mission_uri = mission_base_uri + '?filter={{"robot_id": "{robot_id}"}}'.format(robot_id=robot_id)
        get_mission_resp = requests.get(get_mission_uri, headers=headers, timeout=5)
        if get_mission_resp.status_code != 200:
             print(bcolors.FAIL + "Error in retrieving missions" + bcolors.ENDC)
             raise SystemExit(1)
        mission_list = json.loads(get_mission_resp.content.decode())['data']
        return mission_list
    
    def create_mission(self, mission_details, headers, tries=0):
        create_mission_uri = self.api_uri + "mission/"
        create_mission_resp = requests.post(create_mission_uri, json=mission_details, headers=headers, timeout=5)
        if create_mission_resp.status_code != 200:
            if 'UniqueViolation' in json.loads(create_mission_resp.content.decode())['message'] and tries < 50: 
                return self.create_mission(mission_details, headers, tries)
            else:
                raise Exception(bcolors.FAIL + "Error in creating mission" + bcolors.ENDC)
        return json.loads(create_mission_resp.content.decode())['mission_id']
    
    def delete_mission(self, mission_id, headers):
        delete_mission_uri = self.api_uri + "mission/{mission_id}".format(mission_id=mission_id)
        delete_mission_resp = requests.delete(delete_mission_uri, headers=headers, timeout=5)
        if delete_mission_resp.status_code != 200:
            print(bcolors.FAIL + "Failed to delete mission {mission_id}. Please manually delete it".format(mission_id=mission_id) + bcolors.ENDC)
            return None
        else:
            print("successfully deleted mission {mission_id}".format(mission_id=mission_id))
            return json.loads(delete_mission_resp.content.decode()).get('mission_id', None)
    
    def get_mission_instances(self, mission_id, headers):
        mission_instance_base_uri = self.api_uri + "mission-instance/"
        get_mission_instance_uri = mission_instance_base_uri + '?filter={{"mission_id": "{mission_id}"}}'.format(mission_id=mission_id)
        get_mission_instance_resp = requests.get(get_mission_instance_uri, headers=headers, timeout=5)
        if get_mission_instance_resp.status_code != 200:
            print(bcolors.FAIL + "Error retrieving mission instance" + bcolors.ENDC)
            raise SystemExit(1)
        return json.loads(get_mission_instance_resp.content.decode())['data']

    def create_mission_instance(self, mission_instance_details, headers):
        create_mission_instance_uri = self.api_uri + "mission-instance/"
        create_mission_instance_resp = requests.post(create_mission_instance_uri, json=mission_instance_details, headers=headers)
        if create_mission_instance_resp.status_code != 200:
            raise Exception(bcolors.FAIL + "Error in creating mission instance" + bcolors.ENDC)
        return json.loads(create_mission_instance_resp.content.decode())['mission_instance_id']

    def delete_mission_instance(self, mission_instance_id, headers):
        delete_mission_instance_uri = self.api_uri + "mission-instance/" + str(mission_instance_id)
        delete_mission_instance_resp = requests.delete(delete_mission_instance_uri, headers=headers, timeout=5)
        if delete_mission_instance_resp.status_code != 200:
            print(bcolors.FAIL + "Failed to delete mission_instance {mission_instance_id}. Please manually delete it".format(mission_instance_id=mission_instance_id) + bcolors.ENDC)
        else:
            print("successfully deleted mission_instance {mission_instance_id}".format(mission_instance_id=mission_instance_id))
    
    def get_waypoints_from_map_id(self, map_id, headers):
        waypoint_base_uri = self.api_uri + "waypoint/"
        get_waypoint_uri = waypoint_base_uri + '?filter={{"map_id": "{map_id}"}}'.format(map_id=map_id)
        get_waypoint_resp = requests.get(get_waypoint_uri, headers=headers, timeout=5)
        if get_waypoint_resp.status_code != 200:
            print(bcolors.FAIL + "Error retrieving waypoint" + bcolors.ENDC)
            raise SystemExit(1)
        return json.loads(get_waypoint_resp.content.decode())['data']

    def create_waypoint(self, waypoint_details, headers):
        waypoint_base_uri = self.api_uri + "waypoint/"
        create_waypoint_uri = waypoint_base_uri
        create_waypoint_resp = requests.post(create_waypoint_uri, json=waypoint_details, headers=headers, timeout=5)
        if create_waypoint_resp.status_code != 200:
            raise Exception(bcolors.FAIL + "Error in creating waypoint" + bcolors.ENDC)
        return json.loads(create_waypoint_resp.content.decode())['waypoint_id']
    
    def delete_waypoint(self, waypoint_id, headers):
        delete_waypoint_uri = self.api_uri + "waypoint/"
        delete_waypoint_uri += str(waypoint_id)
        delete_waypoint_resp = requests.delete(delete_waypoint_uri, headers=headers, timeout=5)
        if delete_waypoint_resp.status_code != 200:
            print(bcolors.FAIL + "Failed to delete waypoint {waypoint_id}. Please manually delete it".format(waypoint_id=waypoint_id) + bcolors.ENDC)
        else:
            print("successfully deleted waypoint {waypoint_id}".format(waypoint_id=waypoint_id))
    
    def get_map_image(self, map_id, headers):
        map_base_uri = self.api_uri + "map/"
        get_map_uri = map_base_uri + "{map_id}".format(map_id=map_id)
        get_map_resp = requests.get(get_map_uri, headers=headers, timeout=5)
        if get_map_resp.status_code != 200:
            print(bcolors.FAIL + "Error retrieving map {map_id}".format(map_id=map_id) + bcolors.ENDC)
            raise SystemExit(1)
        try:
            image_file = io.BytesIO(get_map_resp.content)
            image_data = image_file.read()
            b64_string = base64.b64encode(image_data).decode('utf-8')
        except Exception as err:
            print("Failed to decode image file")
            raise SystemExit(1)
        
        return b64_string
    
    def get_maps_from_robot_id(self, robot_id, headers):
        map_base_uri = self.api_uri + "map/"
        get_map_uri = map_base_uri + '?filter={{"robot_id":"{robot_id}"}}'.format(robot_id=robot_id)
        get_map_resp = requests.get(get_map_uri, headers=headers, timeout=5)
        if get_map_resp.status_code != 200:
            print(bcolors.FAIL + "Error retrieving maps associated to robot {robot_id}".format(robot_id=robot_id) + bcolors.ENDC)
            raise SystemExit(1)
        return json.loads(get_map_resp.content.decode())['data']
    
    def create_map(self, map_details, headers):
        create_map_uri = self.api_uri + "map/"
        create_map_resp = requests.post(create_map_uri, json=map_details, headers=headers, timeout=5)
        if create_map_resp.status_code != 200:
            raise Exception(bcolors.FAIL + "Error in creating map" + bcolors.ENDC)
        return json.loads(create_map_resp.content.decode())['map_id']

    def delete_map(self, map_id, headers):
        delete_map_uri = self.api_uri + "map/"
        delete_map_uri += str(map_id)
        delete_map_resp = requests.delete(delete_map_uri, headers=headers, timeout=5)
        if delete_map_resp.status_code != 200:
            print(bcolors.FAIL + "Failed to delete map {map_id}. Please manually delete it".format(map_id=map_id) + bcolors.ENDC)
            return None
        else:
            print("successfully deleted map {map_id}".format(map_id=map_id))
            return json.loads(delete_map_resp.content.decode()).get('map_id', None)

    def get_schedules(self, schedule_id, headers):
        schedule_base_uri = self.api_uri + "schedule/"
        get_schedule_uri = schedule_base_uri + 'metadata/{schedule_id}'.format(schedule_id=schedule_id)
        get_schedule_resp = requests.get(get_schedule_uri, headers=headers, timeout=5)
        if get_schedule_resp.status_code != 200:
            print(bcolors.FAIL + "Error in retrieving schedule {schedule_id}".format(schedule_id=schedule_id) + bcolors.ENDC)
            raise SystemExit(1)
        return json.loads(get_schedule_resp.content.decode())
    
    def create_schedule(self, schedule_details, headers):
        create_schedule_uri = self.api_uri + "schedule/"
        create_schedule_resp = requests.post(create_schedule_uri, json=schedule_details, headers=headers, timeout=5)
        if create_schedule_resp.status_code != 200:
            raise Exception(bcolors.FAIL + "Error in creating schedule. Error: {err}".format(err=json.loads(create_schedule_resp.content.decode()).get('message', '')) + bcolors.ENDC)
        return json.loads(create_schedule_resp.content.decode())['schedule_id']
    
    def delete_schedule(self, schedule_id, headers):
        delete_schedule_uri = self.api_uri + "schedule/" + str(schedule_id)
        delete_schedule_resp = requests.delete(delete_schedule_uri, headers=headers, timeout=5)
        if delete_schedule_resp.status_code != 200:
            print(bcolors.FAIL + "Failed to delete schedule {schedule_id}. Please manually delete it".format(schedule_id=schedule_id) + bcolors.ENDC)
            return None
        else:
            print("successfully deleted schedule {schedule_id}".format(schedule_id=schedule_id))
            return json.loads(delete_schedule_resp.content.decode()).get('schedule_id', None)
    
    def get_property_by_id(self, property_id, headers):
        property_base_uri = self.api_uri + "property/"
        get_property_uri = property_base_uri + "{property_id}".format(property_id=property_id)
        get_property_resp = requests.get(get_property_uri, headers=headers, timeout=5)
        if get_property_resp.status_code != 200:
            print(bcolors.FAIL + "Error in retrieving property {property_id} of robot".format(property_id=property_id) + bcolors.ENDC)
            raise SystemExit(1)
        return json.loads(get_property_resp.content.decode())
    
    def get_property_by_filter(self, filters, headers):
        property_base_uri = self.api_uri + "property/"
        filter_string = '?filter={'
        index = 0
        for key, value in filters.items():
            filter_string += '"{key}": "{value}"'.format(key=key, value=value)
            if index != len(filters) - 1:
                filter_string += ', '
            index += 1
        filter_string += '}'
        
        get_property_uri = property_base_uri + filter_string
        get_property_resp = requests.get(get_property_uri, headers=headers, timeout=5)
        if get_property_resp.status_code != 200:
            print(bcolors.FAIL + "Error in retrieving property of robot" + bcolors.ENDC)
            raise SystemExit(1)
        get_property_data = json.loads(get_property_resp.content.decode())['data']
        return get_property_data
    
    def create_property(self, property_details, headers):
        property_base_uri = self.api_uri + "property/"
        create_property_resp = requests.post(property_base_uri, json=property_details, headers=headers, timeout=5)
        if create_property_resp.status_code != 200: 
            raise Exception(bcolors.FAIL + "Error in creating property for new organization. Error: {err}"
                            .format(err=json.loads(create_property_resp.content.decode()).get('message', None)) + bcolors.ENDC)
        return json.loads(create_property_resp.content.decode())['property_id'] 

    def delete_property(self, property_id, headers):
        delete_property_uri = self.api_uri + "property/{property_id}".format(property_id=property_id)
        delete_property_resp = requests.delete(delete_property_uri, headers=headers, timeout=5)
        if delete_property_resp.status_code != 200:
            print(bcolors.FAIL + "Failed to delete property {property_id}. Please manually delete it".format(property_id=property_id) + bcolors.ENDC)
            return None
        else:
            print("successfully deleted property {property_id}".format(property_id=property_id))
            return json.loads(delete_property_resp.content.decode()).get('property_id', None)
        
    def get_robot_config(self, org_id, robot_id, headers):
        get_config_uri = self.api_uri + f'robot_config/config/{org_id}/{robot_id}'
        get_config_resp = requests.get(get_config_uri, headers=headers, timeout=5)
        if get_config_resp.status_code != 200:
            print(get_config_resp.content)
            print(bcolors.FAIL + "Failed to fetch new robot config file" + bcolors.ENDC)
            return None
        return json.loads(get_config_resp.content.decode())

    def switch_org(self, org_id, headers):
        switch_org_uri = self.api_uri + "user/switch_org"
        switch_org_payload = {
             'organization_id': org_id,
             "blacklist": True
        }   
        switch_org_resp = requests.post(switch_org_uri, json=switch_org_payload, headers=headers, timeout=5)
        if switch_org_resp.status_code != 200:
            print(bcolors.FAIL + "Error in switching organization" + bcolors.ENDC)
            raise SystemExit(1)
        new_access_token = json.loads(switch_org_resp.content.decode())['access_token']
        return new_access_token
    
    def rollback(self, created_data, headers):
        for key, value in created_data.items():
            if key == 'property_id':
                self.delete_property(value, headers)
            if key == 'robot':
                if value.get('robot_id', None):
                    self.delete_robot(robot_id=value.get('robot_id'), organization_id=value.get('organization_id', ''), headers=headers)
            if key == 'missions':
                for mission_id in value:
                    self.delete_mission(mission_id, headers=headers)
            if key == 'waypoints':
                for waypoint_id in value:
                    self.delete_waypoint(waypoint_id, headers=headers)
            if key == 'schedules':
                for schedule_id in value:
                    self.delete_schedule(schedule_id, headers=headers)
            if key == 'maps':
                for map_id in value:
                    if map_id:
                        self.delete_map(map_id, headers=headers)
            if key == 'mission_instances':
                for mission_instance_id in value:
                    self.delete_mission_instance(mission_instance_id, headers=headers)

    def move(self, args):
        """
        Main entrypoint to move a robot from one organization to another.

                Parameters:
                        args: populated argument namespace returned by `argparse.parse_args()`
        """
        if not hasattr(args.config, "config") or (hasattr(args.config, "config") and not args.config.config.get("COGNICEPT_USER_API_URI", None)): 
            print(bcolors.FAIL + "Failed to load API URL. Please contact our tech support team for assistance" + bcolors.ENDC)
            raise SystemExit(1)

        self.api_uri = args.config.config.get("COGNICEPT_USER_API_URI")

        if "/v1" in self.api_uri:
            print(bcolors.FAIL + "The cognicept move command is not available with version 1 of cognicept's api. Please contact our tech support team for assistance. " + bcolors.ENDC)
            raise SystemExit(1)
        
        if args.robot_id is None and args.robot_code is None:
            print("Please provide a robot id or robot code")
            raise SystemExit(1)
        elif args.old_org_id is None and args.old_org_code is None:
            print("Please provide the old organization id or code you wish to move the robot from")
            raise SystemExit(1)
        elif args.new_org_id is None and args.new_org_code is None:
            print("Please provide the new organization id or code you wish to move the robot to")
            raise SystemExit(1)
            
        if args.robot_id and args.robot_code:
            print("Please only provide either robot_id or robot_code")
            raise SystemExit(1)

        if (args.old_org_code and args.old_org_id) or (args.new_org_code and args.new_org_id):
            print("Please only provide organization details either as id or org_code")
            raise SystemExit(1)
        
        skip_schedule = False
        skip_map = False
        skip_waypoint = False
        if args.skip:
            arguments = args.skip
            for argument in arguments:
                if argument not in ['schedule', 'map', 'waypoint']:
                    print('Arguments for skip are invalid. Accepted arguments are schedule, map and waypoint')
                    raise SystemExit(1)
                if argument == 'schedule':
                    skip_schedule = True
                elif argument == 'map':
                    skip_map = True
                else:
                    skip_waypoint = True
    
        print(bcolors.HEADER + "Please login to your smart+ account to perform this operation" + bcolors.ENDC)

        username = input('Username: ')
        password = getpass.getpass('Password: ')

        access_token = self.login(username=username, password=password)
        if access_token is None:
             print(bcolors.FAIL + "Error in loading credentials. Please initialise the robot by running robot init" + bcolors.ENDC)
             raise SystemExit(1)
        headers = {
            'Authorization': 'Bearer ' + access_token
        }


        decode_json = jwt.decode(access_token, verify=False, algorithms=["HS256"])
        # False means 2FA Verification is activated by the user
        if decode_json['user_claims']['authorized'] == False:
            new_key = self.mfa_verfication(auth_key=access_token, headers=headers)
            if new_key == access_token:    
                print("Failed to move robot: Invalid OTP entered 3 times")
                raise SystemExit(1)
            else:
                access_token=new_key
                headers = {
                    'Authorization': 'Bearer ' + access_token
                }
        

        print(bcolors.OKBLUE + "Successfully logged in! Proceeding to fetch robot data..." + bcolors.ENDC)

        #### Check if both organizations exist ####
        if args.old_org_id:
            old_org_data = self.get_organization_by_id(org_id=args.old_org_id, headers=headers)
            args.old_org_code = old_org_data['organization_code']
        else:
            old_org_data = self.get_organization_by_org_code(org_code=args.old_org_code, headers=headers)
            args.old_org_id = old_org_data['organization_id']
        
        if args.new_org_id:
            new_org_data = self.get_organization_by_id(org_id=args.new_org_id, headers=headers)
            args.new_org_code = new_org_data['organization_code']
        else:
            new_org_data = self.get_organization_by_org_code(org_code=args.new_org_code, headers=headers)
            args.new_org_id = new_org_data['organization_id']


        # Switch to old_org_id if the default logged in org is not old_org_id #
        access_token = self.switch_org(args.old_org_id, headers=headers)
        headers = {
            'Authorization': 'Bearer ' + access_token
        }

        #### Check if robot exists in old organization ####
        if args.robot_id:
            old_robot_data = self.get_robot_by_id(robot_id=args.robot_id, org_id=args.old_org_id, headers=headers)
            args.robot_code = old_robot_data.get('robot_code', '')
        else:
            old_robot_data = self.get_robot_by_code(robot_code=args.robot_code, org_code=args.old_org_code, headers=headers)
            args.robot_id = old_robot_data.get('robot_id', '')

        print(bcolors.OKBLUE + "Successfully retrieved the old robot. Trying to move the robot to the new organization..." + bcolors.ENDC)

        #### Retrieve Missions associated with robot in old organization #####
        old_mission_list = self.get_robot_missions(robot_id=args.robot_id, headers=headers)

        #### Retrieve all maps associated with robot and waypoints associated to maps - including those not associated to a mission #### 
        map_dict = {}
        waypoint_dict = {}

        if not skip_map:
            maps = self.get_maps_from_robot_id(robot_id=args.robot_id, headers=headers)
            for map in maps:
                map_id = map.get('map_id', None)
                map_image = self.get_map_image(map_id, headers=headers)
                map['map_image'] = 'data:image/png;base64,' + str(map_image)
                if map_id is not None:
                    map_dict[map_id] = map
                
                if not skip_waypoint:
                    waypoints = self.get_waypoints_from_map_id(map_id, headers=headers)
                    for waypoint in waypoints:
                        waypoint_id = waypoint.get('waypoint_id', '')
                        waypoint_dict[waypoint_id] = waypoint

        #### Retrieve mission instances and schedules associated with robot in old organization #####
        if not skip_schedule:
            for mission in old_mission_list:
                mission_id = mission['mission_id']
                schedules = mission['schedules']
        
                mission_instances = self.get_mission_instances(mission_id=mission_id, headers=headers)
                mission_instances_without_schedules = []
                mission_instances_with_schedules = []
                
                if len(mission_instances) > 0 and mission_instances is not None:
                    mission_instances_dict = {}
                    for mission_instance in mission_instances:
                        schedule_id = mission_instance['schedule_id']
                        if schedule_id is None:
                            mission_instances_without_schedules.append(mission_instance)
                        else:
                            mission_instances_dict[schedule_id] = mission_instance
                    mission['schedule_to_mission_instance_dict'] = mission_instances_dict  
                    mission['mission_instances_without_schedules'] = mission_instances_without_schedules

                for index, schedule in enumerate(schedules):
                    schedule_id = schedule['schedule_id']
                    schedules[index] = self.get_schedules(schedule_id=schedule_id, headers=headers)
               
        #### Retrieve property id ####
        old_property_id = old_robot_data['property_id']
        old_property_data = self.get_property_by_id(property_id=old_property_id, headers=headers)

        new_property_id = None
        if old_property_data['phone_number'] is not None and old_property_data is not None:
            # Assumption - phone number and address are unique identifiers of a property 
            # if the property exists within the new organization then use that property id. Else create a new property
            check_property_data = self.get_property_by_filter(filters={"phone_number": old_property_data['phone_number'], "address": old_property_data['address']}, headers=headers)
            if len(check_property_data) == 1:
                new_property_id = check_property_data['data'][0]['property_id']
    

        #### Switch to new organization ####
        new_access_token = self.switch_org(args.new_org_id, headers)
        headers['Authorization'] = 'Bearer ' + new_access_token
        
        newly_created_data = {}

        try:
            #### Create property in new organization - if required ####
            if new_property_id is None:
                new_property_data = {
                    "property_code": old_property_data['property_code'],
                    "property_name": old_property_data['property_name'],
                    "status": "Active",
                    "zendesk_id": old_property_data['zendesk_id']
                }
            
                new_property_id = self.create_property(property_details=new_property_data, headers=headers)
                newly_created_data['property_id'] = new_property_id

            new_robot_data = {
                "model_id": old_robot_data['model_id'],
                "nick_name": old_robot_data['nick_name'],
                "oem_id": old_robot_data['oem_id'],
                "property_id": new_property_id,
                "status": "Offline",
                "zendesk_id": old_robot_data['zendesk_id']
            }

            new_robot_id = self.create_robot(robot_details=new_robot_data, headers=headers)
            newly_created_data['robot'] = {
                "robot_id": new_robot_id,
                "organization_id": args.new_org_id
            }
            
            print(bcolors.OKBLUE + "Porting over robot details..." + bcolors.ENDC)

            #### Create maps ####     
            map_count = 0
            to_delete_map_id = []
            old_to_new_map_dict = {}
            for map_id, map_data in map_dict.items():
                # Prep data for creating map
                old_map_id = map_data.pop('map_id', None)
                to_delete_map_id.append(old_map_id)
                
                map_data['robot_id'] = new_robot_id
                new_map_id = self.create_map(map_details=map_data, headers=headers) 
                old_to_new_map_dict[map_id] = new_map_id
                if new_map_id is not None:
                        maps = newly_created_data.get('maps', [])
                        maps.append(new_map_id)
                        map_count += 1
                
                # log created map ids in case of rollback
                maps = newly_created_data.get('maps', [])
                maps.append(new_map_id)
                newly_created_data['maps'] = maps
                     
            #### Create waypoints - alternative ####
            waypoint_count = 0
            to_delete_waypoint_id = []
            for waypoint_id, waypoint_data in waypoint_dict.items():
                old_waypoint_id = waypoint_data.pop('waypoint_id', None)
                to_delete_waypoint_id.append(old_waypoint_id)

                old_map_id = waypoint_data.pop('map_id', None)
                waypoint_data['map_id'] = old_to_new_map_dict[old_map_id]
                waypoint_data.pop('map', None)
                waypoint_data['waypoint_name'] = waypoint_data.get('name', '')
                waypoint_data.pop('name', None)
                if waypoint_data['tag'] == None:
                    waypoint_data['tag'] = []
                new_waypoint_id = self.create_waypoint(waypoint_details=waypoint_data, headers=headers)
                waypoint_count += 1
                
                created_waypoints = newly_created_data.get('waypoints', [])
                created_waypoints.append(new_waypoint_id)
                newly_created_data['waypoints'] = created_waypoints

                waypoint_data['waypoint_id'] = new_waypoint_id
            
            # associate newly created waypoints with missions #
            for mission in old_mission_list:
                tasks = mission.get('mission_json')['tasks']

                # Used to exclude missions with waypoints - only applicable when skip_waypoint or skip_map is True
                indexes_to_del = []
                curr_index = 0
                for task in tasks:
                    if task.get('parameter', {}).get('waypoint_id', None):
                        if skip_waypoint or skip_map:
                            indexes_to_del.append(curr_index)
                        else:
                            old_waypoint_id = task['parameter']['waypoint_id']
                            task['parameter']['waypoint_id'] = waypoint_dict[old_waypoint_id]['waypoint_id']
                        curr_index += 1

                if skip_waypoint or skip_map:
                    count = 0
                    for idx in indexes_to_del: 
                        del tasks[idx-count]
                        count += 1

            #### Create mission, schedule and mission instances ####     
            schedule_count = 0
            mission_count = 0
            mission_instance_count = 0
            old_mission_ids = []
            for mission in old_mission_list:
                mission_data = mission
                old_mission_id = mission_data.pop('mission_id', None)
                old_mission_ids.append(old_mission_id)
                tasks = mission_data.pop('mission_json')['tasks']
                schedules = mission_data.pop('schedules', {})
                schedule_to_mission_instance_dict = mission_data.pop('schedule_to_mission_instance_dict', {})
                mission_instances_without_schedules = mission_data.pop('mission_instances_without_schedules', [])

                mission_data['robot_id'] = new_robot_id

                if skip_waypoint or skip_map:
                    new_task_list = []
                    for task in tasks:
                        # If task requires waypoint then skip
                        if task.get('type', '') == 'NAV2POINT' or task.get('type', '') == 'MOV2POINT':
                            continue
                        else:
                            new_task_list.append(task)
                    tasks = new_task_list

                mission_data['task_list'] = tasks                    

                new_mission_id = self.create_mission(mission_details=mission_data, headers=headers)
                mission_count += 1
                new_missions = newly_created_data.get('missions', [])
                new_missions.append(new_mission_id)
                newly_created_data['missions'] = new_missions

                if not skip_schedule:
                    for schedule in schedules:
                        old_schedule_id = schedule.pop('schedule_id', None)
                        schedule_json = schedule.pop('schedule_json', None)
                        if schedule_json is not None:
                            timestamp = schedule_json.get('timestamp', None)
                            schedule_cron = schedule_json.get('cron', None)

                            if timestamp is not None:
                                schedule['schedule_timestamp'] = timestamp
                            else:
                                schedule['schedule_cron'] = schedule_cron

                            schedule['loop_count'] = schedule_json.get('loop_count', None)

                        schedule['mission_id'] = new_mission_id
                        if '.' in schedule['start_timestamp']:
                            schedule['start_timestamp'] = schedule['start_timestamp'].split('.')[0] 
                        new_schedule_id = self.create_schedule(schedule_details=schedule, headers=headers)
                        schedule_count += 1
                        schedules = newly_created_data.get('schedules', [])
                        schedules.append(new_schedule_id)
                        newly_created_data['schedules'] = schedules

                        mission_instance = schedule_to_mission_instance_dict.get(old_schedule_id, None)
                        if mission_instance is not None:
                            mission_instance['mission_id'] = new_mission_id
                            mission_instance['schedule_id'] = new_schedule_id
                            mission_instance['robot_id'] = new_robot_id
                        schedule_to_mission_instance_dict[old_schedule_id] = mission_instance

                    for key, mission_instance in schedule_to_mission_instance_dict.items():
                        if mission_instance is None:
                            continue
                        mission_instance.pop("last_updated_at", None)
                        mission_instance.pop("mission_instance_id", None)
                        mission_instance.pop("waypoints", None)
                        # mission_instance.get("schedule_id", None) = null if mission_instance['schedule_id'] = None or there is no schedule_id
                        schedule_id = mission_instance.get("schedule_id", None)
                        if schedule_id is None:
                            mission_instance.pop("schedule_id", None)

                        new_mission_instance_id = self.create_mission_instance(mission_instance_details=mission_instance, headers=headers)
                        mission_instance_count += 1
                        new_mission_instances = newly_created_data.get('mission_instances', [])
                        new_mission_instances.append(new_mission_instance_id)
                        newly_created_data['mission_instances'] = new_mission_instances

                    for mission_instance in mission_instances_without_schedules:
                        mission_instance.pop("last_updated_at", None)
                        mission_instance.pop("mission_instance_id", None)
                        mission_instance.pop("waypoints", None)
                        mission_instance.pop("schedule_id", None)
                        mission_instance['robot_id'] = new_robot_id
                        mission_instance['mission_id'] = new_mission_id

                        new_mission_instance_id = self.create_mission_instance(mission_instance_details=mission_instance, headers=headers)
                        mission_instance_count += 1
                        new_mission_instances = newly_created_data.get('mission_instances', [])
                        new_mission_instances.append(new_mission_instance_id)
                        newly_created_data['mission_instances'] = new_mission_instances

        except Exception as err:
            print(bcolors.FAIL + 'Error occurred while porting over robot to new organization. Details: ' + str(err) + bcolors.ENDC)
            print("Preparing to delete any created data...")
            self.rollback(created_data=newly_created_data,headers=headers)
            raise SystemExit(1)
    

        print(bcolors.OKGREEN + "Successfully ported robot to new organization!\n"
            "-------------- Summary ----------------\n"
            "New robot_id: {robot_id}\n"
            "Number of missions moved over: {mission_move_count} \n"
            "Number of mission instances moved over: {mission_instance_count}\n"
            "Number of schedules moved over: {schedule_move_count}\n"
            "Number of maps moved over: {map_move_count}\n"
            "Number of waypoints moved over: {waypoint_move_count}\n"
            .format(robot_id=new_robot_id, mission_move_count=mission_count, schedule_move_count=schedule_count,
                    map_move_count=map_count, waypoint_move_count=waypoint_count, mission_instance_count=mission_instance_count) \
            + bcolors.ENDC)
        
        new_robot_config = self.get_robot_config(org_id=args.new_org_id, robot_id=new_robot_id, headers=headers)
        config = args.config
        if new_robot_config and config:
            for key in new_robot_config:
                config.config[key] = new_robot_config[key]
        Configuration.save_config(config)

        if args.restart:
            # To run cognicept restart -d 
            args.detach = True
            agent_lifecycle = AgentLifeCycle()
            agent_lifecycle.restart(args)

        if args.delete_robot:
            # Should/Can only be done if the shell is connected to the robot because the restart will only work if the docker images are started inside the robot
            access_token = self.switch_org(org_id=args.old_org_id, headers=headers)
            headers = {
                'Authorization': 'Bearer ' + access_token
            }
            self.delete_robot(robot_id=args.robot_id, organization_id=args.old_org_id, headers=headers)
            for mission_id in old_mission_ids:
                if mission_id:
                    self.delete_mission(mission_id=mission_id, headers=headers)
            for map_id in to_delete_map_id:
                self.delete_map(map_id, headers=headers)


from cogniceptshell.robot_actions import RobotActions
from cogniceptshell.common import bcolors
from unittest.mock import patch
import pytest
import json
from uuid import UUID

robot_actions = RobotActions()
robot_actions.api_uri = "https://dev.cognicept.systems/api/web/v2/"

""" Class for mocking fake data"""
class MockConfig():
    config = {
        "COGNICEPT_USER_API_URI": "https://dev.cognicept.systems/api/web/v2/"
    }

class MockArgs():
    config = MockConfig()
    # user details
    username = 'test_move_robot@kabam.ai'
    password = 'Kabam123#'
    access_token = 'askodaslkdmalskmd1231==asd'
    headers = {
        'Authorization': 'Bearer ' + str(access_token)
    }


    # org details
    old_org_id = '0a3ec88a-4fe0-40f6-8c94-5b074eac24b2'
    old_org_code = 'cognicept_systems_125'
    new_org_id = '169ef896-07fb-402d-ac4a-b1cd825e8745'
    new_org_code = 'org_to_move_robot_to_1'
    wrong_organization_id = '0a3ec88a-4fe0-40f6-8c94-5b074eac24b5'
    wrong_organization_code = 'this_is_wrong'

    # robot details
    robot_id = '9d834f2a-5993-4a84-99b4-943d2b4cce29'
    robot_code = 'DoNotMove'
    robot_details = { "robot_name": "Test" }
    wrong_robot_id = '9d834f2a-5993-4a84-99b4-943d2b4cce22'
    wrong_robot_code = 'This_is_wrong'

    # map details
    map_ids = ['f2dae02a-8f73-460a-989d-2540598b3a1a']
    map_image = 'iVBORw0KGgoAAAANSUhEUg=='
    map_details = {"map_image": "asdasd"}
    b64_encoded_map_image = b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR'

    waypoint_id = '2af2cbeb-7b50-40d1-8692-559eda719e74'

    # mission details
    mission_id = 'fe23e302-209d-4bf5-a669-6ecf441511cd'
    wrong_mission_id = 'fe23e302-209d-4bf5-a669-6ecf441511ce'
    mission_details = {"mission_name": "Test"}
    
    # schedule details
    schedule_id = '323fa670-d991-4ba5-871a-668f190f71e2'
    wrong_schedule_id = '323fa670-d991-4ba5-871a-668f190f71e3'
    schedule_details = {"address": "123"}
    
    property_id = '7b4c8067-7c41-41ce-ba43-366cdc69fc2b'
    property_code = 'MY_SITE'
    property_details = {"address": "123"}
    wrong_property_id = '7b4c8067-7c41-41ce-ba43-366cdc69fc22'

mock_args = MockArgs()

@pytest.fixture
def mock_positive_login_request():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"access_token": "123123123123",
                                        "refresh_token": "123123123123"
                                        }).encode()        
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp
    
def test_login_positive(capsys, mock_positive_login_request):
    access_token = robot_actions.login(username=mock_args.username, password=mock_args.password)
    assert access_token == "123123123123"

@pytest.fixture
def mock_negative_login_request():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"message": "Username and password combination not valid"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_login_negative_wrong_cred(capsys, mock_negative_login_request):
    with pytest.raises(SystemExit):
        access_token = robot_actions.login(username=123, password=mock_args.password+'123') 
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Error in logging in: Wrong credentials" + bcolors.ENDC
    print(out, err)
    
@pytest.fixture
def mock_get_organization_by_id_positive():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"data": [{"organization_id": mock_args.old_org_id,
                                    "organization_code": mock_args.old_org_code }]}).encode()        
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_organization_positive(capsys, mock_get_organization_by_id_positive):
    org_details = robot_actions.get_organization_by_id(org_id='correct_org_id', headers=mock_args.headers)
    assert org_details['organization_code'] == mock_args.old_org_code

    org_details = robot_actions.get_organization_by_org_code(org_code='correct_org_code', headers=mock_args.headers)
    assert org_details['organization_id'] == mock_args.old_org_id

@pytest.fixture
def mock_get_organization_by_id_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"data": []}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_organization_by_id_negative(capsys, mock_get_organization_by_id_negative):
    with pytest.raises(SystemExit):
        org_details = robot_actions.get_organization_by_id(org_id=mock_args.wrong_organization_id, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "No such organization {org_id}".format(org_id=mock_args.wrong_organization_id) + bcolors.ENDC
    print(out, err)

@pytest.fixture
def mock_get_organization_by_code_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"data": []}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_organization_by_code_negative(capsys, mock_get_organization_by_code_negative):
    with pytest.raises(SystemExit):
        org_details = robot_actions.get_organization_by_org_code(org_code=mock_args.wrong_organization_code, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + 'No such organization {org_code}'.format(org_code=mock_args.wrong_organization_code) + bcolors.ENDC
    print(out, err)

@pytest.fixture
def mock_get_robot_positive():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"robot_id": mock_args.robot_id, "robot_code": mock_args.robot_code}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_robot_by_id_positive(capsys, mock_get_robot_positive):
    robot_details = robot_actions.get_robot_by_id(mock_args.robot_id, mock_args.old_org_id, mock_args.headers)
    assert robot_details['robot_id'] == mock_args.robot_id
    assert robot_details['robot_code'] == mock_args.robot_code

@pytest.fixture
def mock_get_robot_by_id_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"message": "Bad request. Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_robot_by_id_negative(capsys, mock_get_robot_by_id_negative):    
    with pytest.raises(SystemExit):
        robot_details = robot_actions.get_robot_by_id(mock_args.wrong_robot_id, mock_args.old_org_id, mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "No such robot {robot_id} in organization {org_id}".format(robot_id=mock_args.wrong_robot_id, org_id=mock_args.old_org_id) + bcolors.ENDC
    print(out, err)

def test_get_robot_by_code_positive(capsys, mock_get_robot_positive):
    robot_details = robot_actions.get_robot_by_id(mock_args.robot_id, mock_args.old_org_id, mock_args.headers)
    assert robot_details['robot_id'] == mock_args.robot_id
    assert robot_details['robot_code'] == mock_args.robot_code

@pytest.fixture
def mock_get_robot_by_code_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"message": "Bad request. Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_robot_by_code_negative(capsys, mock_get_robot_by_code_negative):
    with pytest.raises(SystemExit):
        robot_details = robot_actions.get_robot_by_code(mock_args.wrong_robot_code, mock_args.old_org_code, mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "No such robot {robot_code} in organization {org_code}".format(robot_code=mock_args.wrong_robot_code, org_code=mock_args.old_org_code) + bcolors.ENDC
    print(out, err)

@pytest.fixture
def mock_create_robot_positive():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"robot_id": mock_args.robot_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_create_robot_positive(capsys, mock_create_robot_positive):    
    new_robot_id = robot_actions.create_robot(robot_details=mock_args.robot_details, headers=mock_args.headers)
    assert new_robot_id == mock_args.robot_id

@pytest.fixture
def mock_create_robot_negative():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"message": "Bad request"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_create_robot_negative(capsys, mock_create_robot_negative):
    with pytest.raises(Exception) as excinfo:
        robot_actions.create_robot(robot_details=mock_args.robot_details, headers=mock_args.headers)    
    assert str(excinfo.value) == bcolors.FAIL + "Error in creating robot" + bcolors.ENDC

@pytest.fixture
def mock_delete_robot_positive():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"message": mock_args.robot_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_delete_robot_positive(capsys, mock_delete_robot_positive):
    delete_robot_payload = robot_actions.delete_robot(robot_id=mock_args.robot_id, organization_id=mock_args.old_org_id ,headers=mock_args.headers)
    assert delete_robot_payload == mock_args.robot_id

@pytest.fixture
def mock_delete_robot_negative():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"Message": "Invalid key to delete"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_delete_robot_negative(capsys, mock_delete_robot_negative):
    delete_robot_payload = robot_actions.delete_robot(robot_id=mock_args.robot_id, organization_id=mock_args.old_org_id ,headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Error in deleting robot {robot_id}. Please manually delete it".format(robot_id=mock_args.robot_id) + bcolors.ENDC
    print(out, err)

@pytest.fixture
def mock_get_robot_missions_positive():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"data": [{"mission_id": 1}, {"mission_id": 2}, {"mission_id": 3}]}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_robot_missions_positive(capsys, mock_get_robot_missions_positive):
    missions_data = robot_actions.get_robot_missions(robot_id=mock_args.robot_id, headers=mock_args.headers)
    assert len(missions_data) == 3
    
@pytest.fixture
def mock_get_robot_missions_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_get_robot_missions_negative(capsys, mock_get_robot_missions_negative):
    with pytest.raises(SystemExit):
        missions_data = robot_actions.get_robot_missions(mock_args.robot_id, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Error in retrieving missions" + bcolors.ENDC
    
@pytest.fixture
def mock_create_mission_positive():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"mission_id": mock_args.mission_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_create_mission_positive(capsys, mock_create_mission_positive):
    new_mission_id = robot_actions.create_mission(mock_args.mission_details, headers=mock_args.headers)
    assert new_mission_id == mock_args.mission_id

@pytest.fixture
def mock_create_mission_negative():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"message": "Bad input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_create_mission_negative(capsys, mock_create_mission_negative):    
    with pytest.raises(Exception) as excinfo:
        new_mission_id = robot_actions.create_mission(mock_args.mission_details, headers=mock_args.headers)
    assert str(excinfo.value) == bcolors.FAIL + "Error in creating mission" + bcolors.ENDC

@pytest.fixture
def mock_delete_mission_positive():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"mission_id": mock_args.mission_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_delete_mission_positive(capsys, mock_delete_mission_positive):
    delete_payload = robot_actions.delete_mission(mission_id=mock_args.mission_id, headers=mock_args.headers)
    assert delete_payload == mock_args.mission_id

@pytest.fixture
def mock_delete_mission_negative():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_delete_mission_negative(capsys, mock_delete_mission_negative):
    delete_payload = robot_actions.delete_mission(mission_id=mock_args.wrong_mission_id, headers=mock_args.headers)   
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Failed to delete mission {mission_id}. Please manually delete it".format(mission_id=mock_args.wrong_mission_id) + bcolors.ENDC

@pytest.fixture
def mock_get_map_positive():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"data": [{"map_id": mock_args.map_ids[0]}]}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp
        
def test_get_map_positive(capsys, mock_get_map_positive):
    map_details = robot_actions.get_maps_from_robot_id(robot_id=mock_args.robot_id, headers=mock_args.headers)[0]
    assert map_details['map_id'] == mock_args.map_ids[0]

@pytest.fixture
def mock_get_map_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp
        
def test_get_map_negative(capsys, mock_get_map_negative):
    with pytest.raises(SystemExit):
        map_details = robot_actions.get_maps_from_robot_id(mock_args.robot_id, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Error retrieving maps associated to robot {robot_id}".format(robot_id=mock_args.robot_id) + bcolors.ENDC

@pytest.fixture
def mock_get_map_image_positive():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = mock_args.b64_encoded_map_image
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp
        
def test_get_map_image_positive(mock_get_map_image_positive):
    map_image = robot_actions.get_map_image(map_id=mock_args.map_ids[0], headers=mock_args.headers)
    assert map_image == mock_args.map_image

@pytest.fixture
def mock_get_map_image_negative_400():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = mock_args.b64_encoded_map_image
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp
        
def test_get_map_image_negative_400(capsys, mock_get_map_image_negative_400):
    with pytest.raises(SystemExit):
        map_image = robot_actions.get_map_image(mock_args.map_ids[0], headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Error retrieving map {map_id}".format(map_id=mock_args.map_ids[0]) + bcolors.ENDC

@pytest.fixture
def mock_get_map_image_negative_decoding():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = "123"
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp
  
def test_get_map_image_negative_decoding(capsys, mock_get_map_image_negative_decoding):
    with pytest.raises(SystemExit):
        map_image = robot_actions.get_map_image(mock_args.map_ids[0], headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == "Failed to decode image file"

@pytest.fixture
def mock_create_map_positive():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"map_id": mock_args.map_ids[0]}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp
  
def test_create_map_positive(capsys, mock_create_map_positive):
    new_map_id = robot_actions.create_map(mock_args.map_details, headers=mock_args.headers)
    assert new_map_id == mock_args.map_ids[0]

@pytest.fixture
def mock_create_map_negative():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_create_map_negative(capsys, mock_create_map_negative):
    with pytest.raises(Exception) as excinfo:
        new_map_id = robot_actions.create_map(map_details=mock_args.map_details, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert str(excinfo.value) == bcolors.FAIL + "Error in creating map" + bcolors.ENDC
    
@pytest.fixture
def mock_delete_map_positive():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"map_id": mock_args.map_ids[0]}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_delete_map_positive(mock_delete_map_positive):
    delete_payload = robot_actions.delete_map(map_id=mock_args.map_ids[0], headers=mock_args.headers)
    assert delete_payload == mock_args.map_ids[0]

@pytest.fixture
def mock_delete_map_negative():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_delete_map_negative(capsys, mock_delete_map_negative):
    delete_payload = robot_actions.delete_map(map_id=mock_args.map_ids[0], headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Failed to delete map {map_id}. Please manually delete it".format(map_id=mock_args.map_ids[0]) + bcolors.ENDC
    
@pytest.fixture
def mock_get_schedules_positive():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"schedule_id": mock_args.schedule_id, "robot_id": mock_args.robot_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_schedules_positive(capsys, mock_get_schedules_positive):
    schedule_details = robot_actions.get_schedules(mock_args.schedule_id, headers=mock_args.headers)
    assert schedule_details['schedule_id'] == mock_args.schedule_id

@pytest.fixture
def mock_get_schedules_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_get_schedules_negative(capsys, mock_get_schedules_negative):
    with pytest.raises(SystemExit):
        schedule_details = robot_actions.get_schedules(mock_args.wrong_schedule_id, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Error in retrieving schedule {schedule_id}".format(schedule_id=mock_args.wrong_schedule_id) + bcolors.ENDC  
    
@pytest.fixture
def mock_create_schedule_positive():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"schedule_id": mock_args.schedule_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_create_schedule_positive(capsys, mock_create_schedule_positive):
    new_schedule_id = robot_actions.create_schedule(mock_args.schedule_id, headers=mock_args.headers)
    assert new_schedule_id == mock_args.schedule_id

@pytest.fixture
def mock_create_schedule_negative():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"message": "One of cron or timestamp are required"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_create_schedule_negative(capsys, mock_create_schedule_negative):
    schedule_id = mock_args.schedule_id
    with pytest.raises(Exception) as excinfo:
        new_schedule_id = robot_actions.create_schedule(mock_args.schedule_details, headers=mock_args.headers)
    assert str(excinfo.value) == bcolors.FAIL + "Error in creating schedule. Error: {err}".format(err="One of cron or timestamp are required") + bcolors.ENDC 
   
@pytest.fixture
def mock_delete_schedule_positive():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"schedule_id": mock_args.schedule_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_delete_schedule_positive(capsys, mock_delete_schedule_positive):
    delete_payload = robot_actions.delete_schedule(schedule_id=mock_args.schedule_id, headers=mock_args.headers)
    assert delete_payload == mock_args.schedule_id

@pytest.fixture
def mock_delete_schedule_negative():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_delete_schedule_negative(capsys, mock_delete_schedule_negative):
    delete_payload = robot_actions.delete_schedule(schedule_id=mock_args.wrong_schedule_id, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Failed to delete schedule {schedule_id}. Please manually delete it".format(schedule_id=mock_args.wrong_schedule_id) + bcolors.ENDC 
   
@pytest.fixture
def mock_get_property_by_id_positive():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"property_id": mock_args.property_id, "property_code": mock_args.property_code}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_property_by_id_positive(capsys, mock_get_property_by_id_positive):
    property_details = robot_actions.get_property_by_id(mock_args.property_id, headers=mock_args.headers)
    assert property_details['property_id'] == mock_args.property_id

@pytest.fixture
def mock_get_property_by_id_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_get_property_by_id_negative(capsys, mock_get_property_by_id_negative):
    with pytest.raises(SystemExit):
        property_details = robot_actions.get_property_by_id(mock_args.wrong_property_id, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out == bcolors.FAIL + "Error in retrieving property {property_id} of robot".format(property_id=mock_args.wrong_property_id) + bcolors.ENDC + "\n"
    print(out, err)

@pytest.fixture
def mock_get_property_by_filter_positive():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"data": [{"property_id": mock_args.property_id, "property_code": mock_args.property_code}]}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_get_property_by_filter_positive(capsys, mock_get_property_by_filter_positive):
    filters = {"property_name": "My Site"}
    property_details = robot_actions.get_property_by_filter(filters, headers=mock_args.headers)[0]
    
    assert property_details['property_id'] == mock_args.property_id

@pytest.fixture
def mock_get_property_by_filter_negative():
    with patch("cogniceptshell.robot_actions.requests.get") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_get_property_by_filter_negative(capsys, mock_get_property_by_filter_negative):
    filters = {"property_name": "My Site"}
    
    with pytest.raises(SystemExit):
        property_details = robot_actions.get_property_by_filter(filters, headers=mock_args.headers)[0]
    out, err = capsys.readouterr()
    assert out == bcolors.FAIL + "Error in retrieving property of robot" + bcolors.ENDC + "\n"
    print(out, err)

@pytest.fixture
def mock_create_property_positive():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"property_id": mock_args.property_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_create_property_positive(capsys, mock_create_property_positive):
    new_property_id = robot_actions.create_property(mock_args.property_details, headers=mock_args.headers)
    assert new_property_id == mock_args.property_id

@pytest.fixture
def mock_create_property_negative():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_create_property_negative(capsys, mock_create_property_negative):
    with pytest.raises(Exception) as excinfo:
        new_property_id = robot_actions.create_property(mock_args.property_details, headers=mock_args.headers)
    assert str(excinfo.value) == bcolors.FAIL + "Error in creating property for new organization. Error: {err}".format(err="Invalid input") + bcolors.ENDC 

@pytest.fixture
def mock_delete_property_positive():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"property_id": mock_args.property_id}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_delete_property_positive(capsys, mock_delete_property_positive):
    delete_payload = robot_actions.delete_property(property_id=mock_args.property_id, headers=mock_args.headers)
    assert delete_payload == mock_args.property_id

@pytest.fixture
def mock_delete_property_negative():
    with patch("cogniceptshell.robot_actions.requests.delete") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_delete_property_negative(capsys, mock_delete_property_negative):
    delete_payload = robot_actions.delete_property(property_id=mock_args.wrong_property_id, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == bcolors.FAIL + "Failed to delete property {property_id}. Please manually delete it".format(property_id=mock_args.wrong_property_id) + bcolors.ENDC
    print(out, err)

@pytest.fixture
def mock_switch_org_positive():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"access_token": "123123", "refresh_token": "123123"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 200
        yield fake_resp

def test_switch_org_positive(capsys, mock_switch_org_positive):
    new_access_token = robot_actions.switch_org(mock_args.new_org_id, headers=mock_args.headers)
    assert new_access_token == "123123"

@pytest.fixture
def mock_switch_org_negative():
    with patch("cogniceptshell.robot_actions.requests.post") as fake_resp:    
        fake_return_value = json.dumps({"message": "Invalid input"}).encode()
        fake_resp.return_value.content = fake_return_value
        fake_resp.return_value.status_code = 400
        yield fake_resp

def test_switch_org_negative(capsys, mock_switch_org_negative):
    with pytest.raises(SystemExit):
        new_access_token = robot_actions.switch_org(mock_args.new_org_id, headers=mock_args.headers)
    out, err = capsys.readouterr()
    assert out == bcolors.FAIL + "Error in switching organization" + bcolors.ENDC + "\n"
    print(out, err)

def test_move_no_robot_details(capsys):
    args_without_robot_details = MockArgs()
    args_without_robot_details.robot_id = None
    args_without_robot_details.robot_code = None
    with pytest.raises(SystemExit):
        robot_actions.move(args_without_robot_details)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == 'Please provide a robot id or robot code'
    print(out, err)

def test_move_no_old_org_details(capsys):
    args_without_old_organization_details = MockArgs()
    args_without_old_organization_details.old_org_code = None
    args_without_old_organization_details.old_org_id = None
    with pytest.raises(SystemExit):
        robot_actions.move(args_without_old_organization_details)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == "Please provide the old organization id or code you wish to move the robot from"
    print(out, err)

def test_move_no_new_org_details(capsys):
    args_without_new_organization_details = MockArgs()
    args_without_new_organization_details.new_org_code = None
    args_without_new_organization_details.new_org_id = None
    with pytest.raises(SystemExit):
        robot_actions.move(args_without_new_organization_details)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == "Please provide the new organization id or code you wish to move the robot to"
    print(out, err)

def test_move_both_robot_details(capsys):
    args_with_both_robot_details = MockArgs()
    with pytest.raises(SystemExit):
        robot_actions.move(args_with_both_robot_details)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == "Please only provide either robot_id or robot_code"
    print(out, err)

def test_move_both_old_org_details(capsys):
    args_with_both_old_org_details = MockArgs()
    args_with_both_old_org_details.robot_id = None
    with pytest.raises(SystemExit):
        robot_actions.move(args_with_both_old_org_details)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == "Please only provide organization details either as id or org_code"
    print(out, err)

def test_move_both_new_org_details(capsys):
    args_with_both_new_org_details = MockArgs()
    args_with_both_new_org_details.robot_id = None
    args_with_both_new_org_details.old_org_id = None
    with pytest.raises(SystemExit):
        robot_actions.move(args_with_both_new_org_details)
    out, err = capsys.readouterr()
    assert out.replace("\n", "") == "Please only provide organization details either as id or org_code"
    print(out, err)



import os
import pytest
from unittest.mock import MagicMock
from botocore.exceptions import ClientError
from cogniceptshell.configuration import Configuration

def test_pull_config_templates_success(capsys, tmpdir, monkeypatch):
    # 1. Setup tmpdir with a templates folder inside
    templates_folder = tmpdir.mkdir("templates")
    
    # 2. Mock args.config.config and path
    mock_args = MagicMock()
    mock_args.config.config = {
        'ECS_ROBOT_MODEL': 'robot_model'
    }
    mock_args.path = str(tmpdir)
    
    # 3. Mock the list_objects_v2 response
    mock_s3_client = MagicMock()
    mock_s3_client.list_objects_v2.return_value = {
        'Contents': [
            {'Key': 'robot_model/template1.yaml'},
            {'Key': 'robot_model/template2.yml'},
            {'Key': 'robot_model/other_file.txt'}
        ]
    }
    
    # 4. Mock download_file
    def mock_download_file(bucket, key, destination):
        # Simulate file download by creating an empty file in the templates folder
        template_filename = os.path.basename(key)
        destination_path = os.path.join(templates_folder, template_filename)
        open(destination_path, 'a').close()

    mock_s3_client.download_file = mock_download_file
    
    # Patch the necessary objects with the mocks
    monkeypatch.setattr('boto3.client', lambda service: mock_s3_client)
    monkeypatch.setattr('os.path.expanduser', lambda path: str(tmpdir))
    
    # Call the function
    object = Configuration()
    object.pull_config_templates(mock_args)
    
    # Assert the download message
    assert "All robot configuration templates have been successfully downloaded." in capsys.readouterr().out
    
    # Assert the correct files in the templates directory
    assert templates_folder.join("template1.yaml").check(file=True)
    assert templates_folder.join("template2.yml").check(file=True)
    assert not templates_folder.join("other_file.txt").check()


def test_pull_config_templates_list_objects_error(capsys, tmpdir, mocker):
    # 1. Setup tmpdir with a templates folder inside
    templates_folder = tmpdir.mkdir("templates")

    # 2. Mock the necessary objects and functions
    mock_args = mocker.MagicMock()
    mock_args.config.config = {
        'ECS_ROBOT_MODEL': 'robot_model'
    }
    mock_args.path = str(tmpdir)

    mock_s3_client = mocker.MagicMock()
    mocker.patch("boto3.client", return_value=mock_s3_client)

    mock_error = ClientError({'Error': {'Code': 'SomeErrorCode', 'Message': 'SomeErrorMessage'}}, 'ListObjectsV2')
    mock_s3_client.list_objects_v2.side_effect = mock_error

    # Create an instance of Configuration
    config = Configuration()

    # Call the function
    config.pull_config_templates(mock_args)

    # Assert the correct behavior
    assert "Failed to retrieve template config files" in capsys.readouterr().out

    assert not os.listdir(templates_folder)


def test_pull_config_templates_download_file_error(capsys, tmpdir, mocker):
    # 1. Setup tmpdir with a templates folder inside
    templates_folder = tmpdir.mkdir("templates")

    # 2. Mock the necessary objects and functions
    mock_args = mocker.MagicMock()
    mock_args.config.config = {
        'ECS_ROBOT_MODEL': 'robot_model'
    }
    mock_args.path = str(tmpdir)

    mock_s3_client = mocker.MagicMock()
    mocker.patch("boto3.client", return_value=mock_s3_client)

    mock_response = {
        'Contents': [
            {'Key': 'robot_model/template1.yaml'},
            {'Key': 'robot_model/template2.yml'},
            {'Key': 'robot_model/other_file.txt'}
        ]
    }
    mock_s3_client.list_objects_v2.return_value = mock_response

    mock_error = ClientError({'Error': {'Code': 'SomeErrorCode', 'Message': 'SomeErrorMessage'}}, 'DownloadFile')
    mock_s3_client.download_file.side_effect = mock_error

    # Create an instance of Configuration
    config = Configuration()

    # Call the function
    config.pull_config_templates(mock_args)

    # Assert the correct behavior
    assert "Failed to retrieve template config files" in capsys.readouterr().out

    assert not os.listdir(templates_folder)


import os
import pytest
from unittest.mock import patch
from cogniceptshell.agent_life_cycle import AgentLifeCycle

class MockArgs:
    def __init__(self, path):
        self.path = path
        self.config = MockConfig()

class MockConfig:
    def __init__(self):
        self.config = {
            'KEY1': 'value1',
            'KEY2': 'value2'
        }

def test_populate_config_files_success(capsys, tmpdir):
    # Mock the necessary paths and files
    with patch('cogniceptshell.agent_life_cycle.os.path.expanduser') as mock_expanduser:
        mock_expanduser.return_value = str(tmpdir)

        # Create a mock template file
        template_filename = 'template.yaml'
        template_path = tmpdir.join(template_filename)
        with open(template_path, 'w') as template_file:
            template_file.write("key1: ${KEY1}\nkey2: ${KEY2}")

        # Call the function to be tested
        agent_lifecycle = AgentLifeCycle()
        agent_lifecycle.populate_config_files(MockArgs(str(tmpdir)))

        # Verify the new file is created
        assert template_filename in os.listdir(str(tmpdir))

        # Verify variable replacements
        with open(str(tmpdir.join(template_filename)), 'r') as output_file:
            content = output_file.read()
            assert 'key1: value1' in content
            assert 'key2: value2' in content

        # Capture the printed output
        captured = capsys.readouterr()
        printed_output = captured.out.strip()

        # Assert the expected output in the terminal
        assert printed_output == 'Config files populated and copied successfully.'

def test_populate_config_files_no_template_files(capsys, tmpdir):
    # Mock the necessary paths and files
    with patch('cogniceptshell.agent_life_cycle.os.path.expanduser') as mock_expanduser:
        mock_expanduser.return_value = str(tmpdir)

        # Call the function to be tested
        agent_lifecycle = AgentLifeCycle()

        # Verify failure case: No template file exists
        agent_lifecycle.populate_config_files(MockArgs(str(tmpdir)))

        # Capture the printed output
        captured = capsys.readouterr()
        printed_output = captured.out.strip()

        # Assert the expected output in the terminal
        assert 'No templates found in the specifed source folder' in printed_output

def test_populate_config_files_no_output_folder(tmpdir):
    # Mock the necessary paths and files
    with patch('cogniceptshell.agent_life_cycle.os.path.expanduser') as mock_expanduser:
        mock_expanduser.side_effect = lambda path: path.replace('~', str(tmpdir))

        # Create a mock template file
        template_filename = 'template.yaml'
        template_path = tmpdir.join(template_filename)
        with open(template_path, 'w') as template_file:
            template_file.write("key1: ${KEY1}\nkey2: ${KEY2}")

        # Set output folder to a non-existent directory
        non_existent_folder = tmpdir.join('non_existent_folder')

        # Call the function to be tested
        agent_lifecycle = AgentLifeCycle()

        # Verify failure case: Output folder doesn't exist
        with pytest.raises(FileNotFoundError) as exc_info:
            agent_lifecycle.populate_config_files(MockArgs(str(non_existent_folder)))

        # Assert the specific exception is raised
        assert "Specifed target folder for templates does not exist" in str(exc_info.value)

def test_populate_config_files_no_templates_folder(tmpdir, capsys):
    # Mock the necessary paths and files
    with patch('cogniceptshell.agent_life_cycle.os.path.expanduser') as mock_expanduser, \
            patch('cogniceptshell.agent_life_cycle.os.listdir') as mock_listdir:
        # Set the mock return values
        mock_expanduser.return_value = str(tmpdir)
        mock_listdir.side_effect = FileNotFoundError("[Errno 2] No such file or directory: '/path/to/templates'")

        # Call the function to be tested
        agent_lifecycle = AgentLifeCycle()

        # Verify failure case: No templates folder
        with pytest.raises(SystemExit):
            agent_lifecycle.populate_config_files(MockArgs(str(tmpdir)))



import pytest
from unittest.mock import patch
import psutil

from cogniceptshell.agent_life_cycle import AgentLifeCycle as agent

@pytest.fixture
def mock_disk_usage():
    with patch('psutil.disk_usage') as mock_disk_usage:
        mock_disk_usage.return_value.free = 50 * (1024 * 1024 * 1024)  # 50 GB
        yield mock_disk_usage

def test_get_disk_space(mock_disk_usage):
    result = agent.get_disk_space(None)
    assert result == 50



import pytest
from unittest.mock import patch, MagicMock
from cogniceptshell.agent_life_cycle import AgentLifeCycle as agent

@pytest.fixture
def mock_response():
    response = MagicMock()
    response.json.return_value = {'info': {'version': '1.3.2'}}
    return response

@pytest.fixture
def mock_requests_get(mock_response):
    with patch('requests.get') as mock_get:
        mock_get.return_value = mock_response
        yield mock_get

@pytest.fixture
def mock_input_yes():
    with patch('builtins.input', return_value='y'):
        yield

@pytest.fixture
def mock_input_no():
    with patch('builtins.input', return_value='n'):
        yield

@pytest.fixture
def mock_pkg_resources():
    mock_distribution = MagicMock()
    mock_distribution.version = "1.0.2"
    with patch('pkg_resources.require', return_value=[mock_distribution]):
        yield

@pytest.fixture
def mock_os_system():
    with patch('os.system', return_value=0):
        # Set the return value of the mocked os.system to simulate success (0 exit code)
        yield

def test_version_update_skip_true(mock_pkg_resources, mock_requests_get, mock_os_system, mock_input_yes, capsys):
    args = MagicMock(skip=True)
    agent.cognicept_version_update(None, args)
    captured = capsys.readouterr()
    assert "cognicept-shell current version" in captured.out
    assert "Installing Version 1.3.2" in captured.out

def test_version_update_skip_false_input_yes(mock_pkg_resources, mock_requests_get, mock_os_system, mock_input_yes, capsys):
    args = MagicMock(skip=False)
    agent.cognicept_version_update(None, args)
    captured = capsys.readouterr()
    assert "cognicept-shell current version" in captured.out
    assert "Installing Version 1.3.2" in captured.out

def test_version_update_skip_false_input_no(mock_pkg_resources, mock_requests_get,mock_os_system, mock_input_no, capsys):
    args = MagicMock(skip=False)
    agent.cognicept_version_update(None, args)
    captured = capsys.readouterr()
    assert "cognicept-shell was not updated" in captured.out



import json
import uuid
import pytest
import requests
from datetime import datetime
from unittest.mock import Mock, patch
from requests.exceptions import Timeout,TooManyRedirects

from cogniceptshell.agent_life_cycle import AgentLifeCycle

@pytest.fixture
def mock_object():
    # Mock the object instance and its methods
    mock_obj = Mock()
    mock_obj._docker_images = {
        'image1': 'ecr/image1:latest',
        'image2': 'ecr/image2:latest',
    }
    return mock_obj

@pytest.fixture
def mock_args():
    # Mock the args object returned by argparse.parse_args()
    mock_args = Mock()
    mock_args.config.config = {
        'AGENT_POST_API': 'http://example.com/api',
        'AGENT_ID': 'agent123',
        'ROBOT_CODE': 'robot456',
        'SITE_CODE': 'site789',
    }
    return mock_args

def test_successful_update(mock_object, mock_args):
    # Mock the get_version_tag_from_latest method for a successful update
    mock_object.get_version_tag_from_latest.return_value = 'v1.0'

    # Call the function with the mocked objects
    result = True
    AgentLifeCycle._update_event_log(mock_object, result, mock_args)

    # Assert the payload values after the function call
    expected_payload = {
        "agent_id": "",
        "compounding": False,
        "create_ticket": False,
        "description": "Null",
        "error_code": "Null",
        "event_id": "",
        "level": "2",
        "message": "Update is successful! Current images are: image1: 'latest:v1.0' ; image2: 'latest:v1.0'",
        "module": "Updater",
        "property_id": "",
        "resolution": "Null",
        "robot_id": "",
        "source": "auto_updater",
        "timestamp": ""
    }

    assert mock_object.post_event_log.call_count == 1
    actual_payload = mock_object.post_event_log.call_args[0][0]
    assert actual_payload == expected_payload

def test_failed_update(mock_object, mock_args):
    # Mock the get_version_tag_from_latest method for a failed update
    mock_object.get_version_tag_from_latest.return_value = None

    # Call the function with the mocked objects
    result = False
    AgentLifeCycle._update_event_log(mock_object, result, mock_args)

    # Assert the payload values after the function call
    expected_payload = {
        "agent_id": "",
        "compounding": False,
        "create_ticket": True,
        "description": "Null",
        "error_code": "Null",
        "event_id": "",
        "level": "16",
        "message": "Update has failed due to lack of disk space or error in updating images. Current images are: image1: 'latest' ; image2: 'latest'",
        "module": "Updater",
        "property_id": "",
        "resolution": "Null",
        "robot_id": "",
        "source": "auto_updater",
        "timestamp": ""
    }

    assert mock_object.post_event_log.call_count == 1
    actual_payload = mock_object.post_event_log.call_args[0][0]
    assert actual_payload == expected_payload


def test_successful_post(mock_args):
    # Mock the requests.post method for a successful request
    mock_response = Mock()
    mock_response.status_code = 200
    mock_response.json.return_value = {'status': 'success'}

    def mock_post(*args, **kwargs):
        return mock_response

    # Mock the datetime.utcnow and uuid.uuid4 functions
    fixed_timestamp = '2023-07-24T13:09:42.434015'
    fixed_event_id = '7fca97c9-17b3-488c-815f-b62adce155d7'
    with patch('requests.post', side_effect=mock_post) as mock_post_request, \
         patch('cogniceptshell.agent_life_cycle.datetime') as mock_datetime, \
         patch('cogniceptshell.agent_life_cycle.uuid') as mock_uuid:
        
        # Configure the mock datetime and uuid
        mock_datetime.utcnow.return_value = datetime.fromisoformat(fixed_timestamp)
        mock_uuid.uuid4.return_value = uuid.UUID(fixed_event_id)
        
        # Call the function with the mocked objects
        payload = {
            "agent_id": "",
            "compounding": False,
            "create_ticket": False,
            "description": "Null",
            "error_code": "Null",
            "event_id": "",
            "level": "2",
            "message": "Update is successful! Current images are: image1: 'latest' ; image2: 'latest'",
            "module": "Updater",
            "property_id": "",
            "resolution": "Null",
            "robot_id": "",
            "source": "auto_updater",
            "timestamp": ""
        }
        agent = AgentLifeCycle()
        result = agent.post_event_log(payload, mock_args)

        # Define the expected payload format
        expected_payload = {
            'agent_id': 'agent123',
            'compounding': False,
            'create_ticket': False,
            'description': 'Null',
            'error_code': 'Null',
            'event_id': fixed_event_id,
            'level': '2',
            'message': "Update is successful! Current images are: image1: 'latest' ; image2: 'latest'",
            'module': 'Updater',
            'property_id': 'site789',
            'resolution': 'Null',
            'robot_id': 'robot456',
            'source': 'auto_updater',
            'timestamp': fixed_timestamp
        }

        # Assert the result and compare the payload format
        assert result is True
        # Ensure the 'data' argument contains the correct payload
        _, actual_args = mock_post_request.call_args  # Get the call arguments of requests.post
        actual_payload = actual_args['json']  # Convert 'data' back to a dictionary
        assert actual_payload == expected_payload

def test_timeout_exception(mock_args):
    # Mock the requests.post method to raise Timeout exception
    def mock_post(*args, **kwargs):
        raise Timeout

    requests.post = mock_post

    # Call the function with the mocked objects
    payload = {
        'message': 'Test message',
        'level': '2',
    }
    agent = AgentLifeCycle()
    result = agent.post_event_log(payload, mock_args)

    # Assert the result
    assert result is False

def test_redirects_exception(mock_args):
    # Mock the requests.post method to raise TooManyRedirects exception
    def mock_post(*args, **kwargs):
        raise TooManyRedirects

    requests.post = mock_post

    # Call the function with the mocked objects
    payload = {
        'message': 'Test message',
        'level': '2',
    }
    agent = AgentLifeCycle()
    result = agent.post_event_log(payload, mock_args)

    # Assert the result
    assert result is False

def test_other_exceptions(mock_args):
    # Mock the requests.post method to raise other exceptions
    def mock_post(*args, **kwargs):
        raise ValueError('Some error occurred')

    requests.post = mock_post

    # Call the function with the mocked objects
    payload = {
        'message': 'Test message',
        'level': '2',
    }
    agent = AgentLifeCycle()
    result = agent.post_event_log(payload, mock_args)

    # Assert the result
    assert result is False



#!/usr/bin/env python
import re
import ast

from setuptools import setup, find_packages


requires = ['urllib3==1.25.4',
            'python-dotenv==0.14.0',
            'boto3>=1.14.0',
            'docker==5.0.3',
            'PyCryptodome >=3.9.8',
            'argcomplete>=1.12.3',
            'tabulate',
            'ping3',
            'PyJWT==1.7.1',
            'websocket-client==0.59.0',
            'paramiko>=2.4.2',
            'psutil',
            'ruamel.yaml',
            'pyaml']

with open("README.md") as readme_file:
    README = readme_file.read()

setup(
    name="cognicept-shell",
    version="1.7.4",
    description="Shell utility to configure Cognicept tools.",
    long_description_content_type="text/markdown",
    long_description=README,
    author="Jakub Tomasek",
    url="https://kabam.ai",
    packages=find_packages(),
    install_requires=requires,
    license="Apache License 2.0",
    entry_points={
        "console_scripts": ["cognicept=cogniceptshell.interface:main"],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Natural Language :: English",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.10",
    ],
)
